package builtin 0.014;

use strict;
use warnings;

# All code, including &import, is implemented by always-present
# functions in the perl interpreter itself.
# See also `builtin.c` in perl source

1;
__END__

